@@ -63,18 +63,24 @@ class ProxyImage(Image):
     :Events:
         `on_load`
             Fired when the image is loaded or changed.
+        `on_error`
+            Fired when the image cannot be loaded.
+            `error`: Exception data that ocurred
     '''
 
-    __events__ = ('on_load', )
+    __events__ = ('on_load', 'on_error')
 
     def __init__(self, arg, **kwargs):
-        kwargs.setdefault('loaded', False)
+        loaded = kwargs.pop('loaded', False)
         super(ProxyImage, self).__init__(arg, **kwargs)
-        self.loaded = kwargs.get('loaded')
+        self.loaded = loaded
 
     def on_load(self):
         pass
 
+    def on_error(self, error):
+        pass
+
 
 class LoaderBase(object):
     '''Common base for the Loader and specific implementations.
@@ -83,6 +89,7 @@ class LoaderBase(object):
     The _update() function is called every 1 / 25.s or each frame if we have
     less than 25 FPS.
     '''
+    _trigger_update = None
 
     def __init__(self):
         self._loading_image = None
@@ -100,10 +107,8 @@ class LoaderBase(object):
         self._trigger_update = Clock.create_trigger(self._update)
 
     def __del__(self):
-        try:
-            Clock.unschedule(self._update)
-        except Exception:
-            pass
+        if self._trigger_update is not None:
+            self._trigger_update.cancel()
 
     def _set_num_workers(self, num):
         if num < 2:
@@ -255,7 +260,7 @@ class LoaderBase(object):
         try:
             proto = filename.split(':', 1)[0]
         except:
-            #if blank filename then return
+            # if blank filename then return
             return
         if load_callback is not None:
             data = load_callback(filename)
@@ -293,7 +298,7 @@ class LoaderBase(object):
         if proto == 'smb':
             try:
                 # note: it's important to load SMBHandler every time
-                # otherwise the data is occasionaly not loaded
+                # otherwise the data is occasionally not loaded
                 from smb.SMBHandler import SMBHandler
             except ImportError:
                 Logger.warning(
@@ -344,13 +349,24 @@ class LoaderBase(object):
             # FIXME create a clean API for that
             for imdata in data._data:
                 imdata.source = filename
-        except Exception:
+        except Exception as ex:
             Logger.exception('Loader: Failed to load image <%s>' % filename)
             # close file when remote file not found or download error
             try:
-                close(_out_osfd)
+                if _out_osfd:
+                    close(_out_osfd)
             except OSError:
                 pass
+
+            # update client
+            for c_filename, client in self._client[:]:
+                if filename != c_filename:
+                    continue
+                # got one client to update
+                client.image = self.error_image
+                client.dispatch('on_error', error=ex)
+                self._client.remove((c_filename, client))
+
             return self.error_image
         finally:
             if fd:
@@ -450,10 +466,14 @@ class LoaderBase(object):
 
         return client
 
+    def remove_from_cache(self, filename):
+        Cache.remove('kv.loader', filename)
+
 #
 # Loader implementation
 #
 
+
 if 'KIVY_DOC' in environ:
 
     Loader = None
