@@ -5,7 +5,7 @@ Adapter tests
 
 import unittest
 
-from kivy.uix.listview import SelectableView
+from kivy.uix.selectableview import SelectableView
 from kivy.uix.listview import ListItemButton
 from kivy.uix.listview import ListItemLabel
 from kivy.uix.listview import CompositeListItem
@@ -73,83 +73,83 @@ fruit_categories = \
                           'is_selected': False}}
 
 fruit_data_list_of_dicts = \
-    [{'name':'Apple',
+    [{'name': 'Apple',
       'Serving Size': '1 large (242 g/8 oz)',
       'data': [130, 0, 0, 0, 0, 0, 260, 7, 34, 11, 5, 20, 25, 1, 2, 8, 2, 2],
       'is_selected': False},
-     {'name':'Avocado',
+     {'name': 'Avocado',
       'Serving Size': '1/5 medium (30 g/1.1 oz)',
       'data': [50, 35, 4.5, 7, 0, 0, 140, 4, 3, 1, 1, 4, 0, 1, 0, 4, 0, 2],
       'is_selected': False},
-     {'name':'Banana',
+     {'name': 'Banana',
       'Serving Size': '1 medium (126 g/4.5 oz)',
       'data': [110, 0, 0, 0, 0, 0, 450, 13, 30, 10, 3, 12, 19, 1, 2, 15, 0, 2],
       'is_selected': False},
-     {'name':'Cantaloupe',
+     {'name': 'Cantaloupe',
       'Serving Size': '1/4 medium (134 g/4.8 oz)',
       'data': [50, 0, 0, 0, 20, 1, 240, 7, 12, 4, 1, 4, 11, 1, 120, 80, 2, 2],
       'is_selected': False},
-     {'name':'Grapefruit',
+     {'name': 'Grapefruit',
       'Serving Size': '1/2 medium (154 g/5.5 oz)',
       'data': [60, 0, 0, 0, 0, 0, 160, 5, 15, 5, 2, 8, 11, 1, 35, 100, 4, 0],
       'is_selected': False},
-     {'name':'Grape',
+     {'name': 'Grape',
       'Serving Size': '3/4 cup (126 g/4.5 oz)',
       'data': [90, 0, 0, 0, 15, 1, 240, 7, 23, 8, 1, 4, 20, 0, 0, 2, 2, 0],
       'is_selected': False},
-     {'name':'Honeydew',
+     {'name': 'Honeydew',
       'Serving Size': '1/10 medium melon (134 g/4.8 oz)',
       'data': [50, 0, 0, 0, 30, 1, 210, 6, 12, 4, 1, 4, 11, 1, 2, 45, 2, 2],
       'is_selected': False},
-     {'name':'Kiwifruit',
+     {'name': 'Kiwifruit',
       'Serving Size': '2 medium (148 g/5.3 oz)',
       'data': [90, 10, 1, 2, 0, 0, 450, 13, 20, 7, 4, 16, 13, 1, 2, 240, 4, 2],
       'is_selected': False},
-     {'name':'Lemon',
+     {'name': 'Lemon',
       'Serving Size': '1 medium (58 g/2.1 oz)',
       'data': [15, 0, 0, 0, 0, 0, 75, 2, 5, 2, 2, 8, 2, 0, 0, 40, 2, 0],
       'is_selected': False},
-     {'name':'Lime',
+     {'name': 'Lime',
       'Serving Size': '1 medium (67 g/2.4 oz)',
       'data': [20, 0, 0, 0, 0, 0, 75, 2, 7, 2, 2, 8, 0, 0, 0, 35, 0, 0],
       'is_selected': False},
-     {'name':'Nectarine',
+     {'name': 'Nectarine',
       'Serving Size': '1 medium (140 g/5.0 oz)',
       'data': [60, 5, 0.5, 1, 0, 0, 250, 7, 15, 5, 2, 8, 11, 1, 8, 15, 0, 2],
       'is_selected': False},
-     {'name':'Orange',
+     {'name': 'Orange',
       'Serving Size': '1 medium (154 g/5.5 oz)',
       'data': [80, 0, 0, 0, 0, 0, 250, 7, 19, 6, 3, 12, 14, 1, 2, 130, 6, 0],
       'is_selected': False},
-     {'name':'Peach',
+     {'name': 'Peach',
       'Serving Size': '1 medium (147 g/5.3 oz)',
       'data': [60, 0, 0.5, 1, 0, 0, 230, 7, 15, 5, 2, 8, 13, 1, 6, 15, 0, 2],
       'is_selected': False},
-     {'name':'Pear',
+     {'name': 'Pear',
       'Serving Size': '1 medium (166 g/5.9 oz)',
       'data': [100, 0, 0, 0, 0, 0, 190, 5, 26, 9, 6, 24, 16, 1, 0, 10, 2, 0],
       'is_selected': False},
-     {'name':'Pineapple',
+     {'name': 'Pineapple',
       'Serving Size': '2 slices, 3" diameter, 3/4" thick (112 g/4 oz)',
       'data': [50, 0, 0, 0, 10, 0, 120, 3, 13, 4, 1, 4, 10, 1, 2, 50, 2, 2],
       'is_selected': False},
-     {'name':'Plum',
+     {'name': 'Plum',
       'Serving Size': '2 medium (151 g/5.4 oz)',
       'data': [70, 0, 0, 0, 0, 0, 230, 7, 19, 6, 2, 8, 16, 1, 8, 10, 0, 2],
       'is_selected': False},
-     {'name':'Strawberry',
+     {'name': 'Strawberry',
       'Serving Size': '8 medium (147 g/5.3 oz)',
       'data': [50, 0, 0, 0, 0, 0, 170, 5, 11, 4, 2, 8, 8, 1, 0, 160, 2, 2],
       'is_selected': False},
-     {'name':'Cherry',
+     {'name': 'Cherry',
       'Serving Size': '21 cherries; 1 cup (140 g/5.0 oz)',
       'data': [100, 0, 0, 0, 0, 0, 350, 10, 26, 9, 1, 4, 16, 1, 2, 15, 2, 2],
       'is_selected': False},
-     {'name':'Tangerine',
+     {'name': 'Tangerine',
       'Serving Size': '1 medium (109 g/3.9 oz)',
       'data': [50, 0, 0, 0, 0, 0, 160, 5, 13, 4, 2, 8, 9, 1, 6, 45, 4, 0],
       'is_selected': False},
-     {'name':'Watermelon',
+     {'name': 'Watermelon',
       'Serving Size': '1/18 medium melon; 2 cups diced pieces (280 g/10.0 oz)',
       'data': [80, 0, 0, 0, 0, 0, 270, 8, 21, 7, 1, 4, 20, 1, 30, 25, 2, 4],
       'is_selected': False}]
@@ -202,26 +202,27 @@ for fruit_record in fruit_data_list_of_dicts:
 
 
 class CategoryItem(SelectableDataItem):
-    def __init__(self, **kwargs):
-        super(CategoryItem, self).__init__(**kwargs)
-        self.name = kwargs.get('name', '')
-        self.fruits = kwargs.get('fruits', [])
-        self.is_selected = kwargs.get('is_selected', False)
+    def __init__(self, is_selected=False, fruits=None, name='', **kwargs):
+        super(CategoryItem, self).__init__(is_selected=is_selected, **kwargs)
+        self.name = name
+        self.fruits = fruits if fruits is not None else []
+        self.is_selected = is_selected
 
 
 class FruitItem(SelectableDataItem):
-    def __init__(self, **kwargs):
-        super(FruitItem, self).__init__(**kwargs)
-        self.name = kwargs.get('name', '')
-        self.serving_size = kwargs.get('Serving Size', '')
-        self.data = kwargs.get('data', [])
-        self.is_selected = kwargs.get('is_selected', False)
+    def __init__(self, is_selected=False, data=None, name='', **kwargs):
+        self.serving_size = kwargs.pop('Serving Size', '')
+        super(FruitItem, self).__init__(is_selected=is_selected, **kwargs)
+        self.name = name
+        self.data = data if data is not None else data
+        self.is_selected = is_selected
 
 
 def reset_to_defaults(db_dict):
     for key in db_dict:
         db_dict[key]['is_selected'] = False
 
+
 category_data_items = \
     [CategoryItem(**fruit_categories[c]) for c in sorted(fruit_categories)]
 
@@ -287,7 +288,7 @@ class AdaptersTestCase(unittest.TestCase):
 
         # The third of the four cls_dict items has no kwargs nor text, so
         # rec['text'] will be set for it. Likewise, the fifth item has kwargs,
-        # but it has no 'text' key/value, so should receive the same treatment.
+        # but it has no 'text' key-value, so should receive the same treatment.
         self.composite_args_converter = \
             lambda row_index, rec: \
                 {'text': rec['text'],
@@ -300,7 +301,7 @@ class AdaptersTestCase(unittest.TestCase):
                                            'is_representing_cls': True}},
                                {'cls': ListItemButton},
                                {'cls': ListItemButton,
-                                'kwargs': {'some key': 'some value'}},
+                                'kwargs': {}},
                                {'cls': ListItemButton,
                                 'kwargs': {'text': rec['text']}}]}
 
@@ -1064,12 +1065,13 @@ class AdaptersTestCase(unittest.TestCase):
         # data records, and the args_converter above that will operate one
         # each item in the data to produce list item view instances from the
         # CompositeListItem class.
-        dict_adapter = DictAdapter(sorted_keys=item_strings,
-                                   data=self.integers_dict,
-                                   args_converter=self.composite_args_converter,
-                                   selection_mode='single',
-                                   allow_empty_selection=False,
-                                   cls=CompositeListItem)
+        dict_adapter = DictAdapter(
+                            sorted_keys=item_strings,
+                            data=self.integers_dict,
+                            args_converter=self.composite_args_converter,
+                            selection_mode='single',
+                            allow_empty_selection=False,
+                            cls=CompositeListItem)
 
         self.assertEqual(len(dict_adapter.selection), 1)
 
@@ -1080,11 +1082,12 @@ class AdaptersTestCase(unittest.TestCase):
 
     # test that sorted_keys is built, if not provided.
     def test_dict_adapter_no_sorted_keys(self):
-        dict_adapter = DictAdapter(data=self.integers_dict,
-                                   args_converter=self.composite_args_converter,
-                                   selection_mode='single',
-                                   allow_empty_selection=False,
-                                   cls=CompositeListItem)
+        dict_adapter = DictAdapter(
+                            data=self.integers_dict,
+                            args_converter=self.composite_args_converter,
+                            selection_mode='single',
+                            allow_empty_selection=False,
+                            cls=CompositeListItem)
 
         self.assertEqual(len(dict_adapter.sorted_keys), 100)
 
@@ -1325,7 +1328,7 @@ class AdaptersTestCase(unittest.TestCase):
         def sorted_keys_ok(letters_dict_adapter):
             sorted_keys_ok = True
             for key in letters_dict_adapter.sorted_keys:
-                if not key in letters_dict_adapter.data:
+                if key not in letters_dict_adapter.data:
                     sorted_keys_ok = False
                     break
             return sorted_keys_ok
