@@ -51,83 +51,83 @@ fruit_categories = \
                           'is_selected': False}}
 
 fruit_data_list_of_dicts = \
-    [{'name':'Apple',
+    [{'name': 'Apple',
       'Serving Size': '1 large (242 g/8 oz)',
       'data': [130, 0, 0, 0, 0, 0, 260, 7, 34, 11, 5, 20, 25, 1, 2, 8, 2, 2],
       'is_selected': False},
-     {'name':'Avocado',
+     {'name': 'Avocado',
       'Serving Size': '1/5 medium (30 g/1.1 oz)',
       'data': [50, 35, 4.5, 7, 0, 0, 140, 4, 3, 1, 1, 4, 0, 1, 0, 4, 0, 2],
       'is_selected': False},
-     {'name':'Banana',
+     {'name': 'Banana',
       'Serving Size': '1 medium (126 g/4.5 oz)',
       'data': [110, 0, 0, 0, 0, 0, 450, 13, 30, 10, 3, 12, 19, 1, 2, 15, 0, 2],
       'is_selected': False},
-     {'name':'Cantaloupe',
+     {'name': 'Cantaloupe',
       'Serving Size': '1/4 medium (134 g/4.8 oz)',
       'data': [50, 0, 0, 0, 20, 1, 240, 7, 12, 4, 1, 4, 11, 1, 120, 80, 2, 2],
       'is_selected': False},
-     {'name':'Grapefruit',
+     {'name': 'Grapefruit',
       'Serving Size': '1/2 medium (154 g/5.5 oz)',
       'data': [60, 0, 0, 0, 0, 0, 160, 5, 15, 5, 2, 8, 11, 1, 35, 100, 4, 0],
       'is_selected': False},
-     {'name':'Grape',
+     {'name': 'Grape',
       'Serving Size': '3/4 cup (126 g/4.5 oz)',
       'data': [90, 0, 0, 0, 15, 1, 240, 7, 23, 8, 1, 4, 20, 0, 0, 2, 2, 0],
       'is_selected': False},
-     {'name':'Honeydew',
+     {'name': 'Honeydew',
       'Serving Size': '1/10 medium melon (134 g/4.8 oz)',
       'data': [50, 0, 0, 0, 30, 1, 210, 6, 12, 4, 1, 4, 11, 1, 2, 45, 2, 2],
       'is_selected': False},
-     {'name':'Kiwifruit',
+     {'name': 'Kiwifruit',
       'Serving Size': '2 medium (148 g/5.3 oz)',
       'data': [90, 10, 1, 2, 0, 0, 450, 13, 20, 7, 4, 16, 13, 1, 2, 240, 4, 2],
       'is_selected': False},
-     {'name':'Lemon',
+     {'name': 'Lemon',
       'Serving Size': '1 medium (58 g/2.1 oz)',
       'data': [15, 0, 0, 0, 0, 0, 75, 2, 5, 2, 2, 8, 2, 0, 0, 40, 2, 0],
       'is_selected': False},
-     {'name':'Lime',
+     {'name': 'Lime',
       'Serving Size': '1 medium (67 g/2.4 oz)',
       'data': [20, 0, 0, 0, 0, 0, 75, 2, 7, 2, 2, 8, 0, 0, 0, 35, 0, 0],
       'is_selected': False},
-     {'name':'Nectarine',
+     {'name': 'Nectarine',
       'Serving Size': '1 medium (140 g/5.0 oz)',
       'data': [60, 5, 0.5, 1, 0, 0, 250, 7, 15, 5, 2, 8, 11, 1, 8, 15, 0, 2],
       'is_selected': False},
-     {'name':'Orange',
+     {'name': 'Orange',
       'Serving Size': '1 medium (154 g/5.5 oz)',
       'data': [80, 0, 0, 0, 0, 0, 250, 7, 19, 6, 3, 12, 14, 1, 2, 130, 6, 0],
       'is_selected': False},
-     {'name':'Peach',
+     {'name': 'Peach',
       'Serving Size': '1 medium (147 g/5.3 oz)',
       'data': [60, 0, 0.5, 1, 0, 0, 230, 7, 15, 5, 2, 8, 13, 1, 6, 15, 0, 2],
       'is_selected': False},
-     {'name':'Pear',
+     {'name': 'Pear',
       'Serving Size': '1 medium (166 g/5.9 oz)',
       'data': [100, 0, 0, 0, 0, 0, 190, 5, 26, 9, 6, 24, 16, 1, 0, 10, 2, 0],
       'is_selected': False},
-     {'name':'Pineapple',
+     {'name': 'Pineapple',
       'Serving Size': '2 slices, 3" diameter, 3/4" thick (112 g/4 oz)',
       'data': [50, 0, 0, 0, 10, 0, 120, 3, 13, 4, 1, 4, 10, 1, 2, 50, 2, 2],
       'is_selected': False},
-     {'name':'Plum',
+     {'name': 'Plum',
       'Serving Size': '2 medium (151 g/5.4 oz)',
       'data': [70, 0, 0, 0, 0, 0, 230, 7, 19, 6, 2, 8, 16, 1, 8, 10, 0, 2],
       'is_selected': False},
-     {'name':'Strawberry',
+     {'name': 'Strawberry',
       'Serving Size': '8 medium (147 g/5.3 oz)',
       'data': [50, 0, 0, 0, 0, 0, 170, 5, 11, 4, 2, 8, 8, 1, 0, 160, 2, 2],
       'is_selected': False},
-     {'name':'Cherry',
+     {'name': 'Cherry',
       'Serving Size': '21 cherries; 1 cup (140 g/5.0 oz)',
       'data': [100, 0, 0, 0, 0, 0, 350, 10, 26, 9, 1, 4, 16, 1, 2, 15, 2, 2],
       'is_selected': False},
-     {'name':'Tangerine',
+     {'name': 'Tangerine',
       'Serving Size': '1 medium (109 g/3.9 oz)',
       'data': [50, 0, 0, 0, 0, 0, 160, 5, 13, 4, 2, 8, 9, 1, 6, 45, 4, 0],
       'is_selected': False},
-     {'name':'Watermelon',
+     {'name': 'Watermelon',
       'Serving Size': '1/18 medium melon; 2 cups diced pieces (280 g/10.0 oz)',
       'data': [80, 0, 0, 0, 0, 0, 270, 8, 21, 7, 1, 4, 20, 1, 30, 25, 2, 4],
       'is_selected': False}]
@@ -179,21 +179,21 @@ for fruit_record in fruit_data_list_of_dicts:
 
 class CategoryItem(SelectableDataItem):
 
-    def __init__(self, **kwargs):
-        super(CategoryItem, self).__init__(**kwargs)
-        self.name = kwargs.get('name', '')
-        self.fruits = kwargs.get('fruits', [])
-        self.is_selected = kwargs.get('is_selected', False)
+    def __init__(self, is_selected=False, fruits=None, name='', **kwargs):
+        super(CategoryItem, self).__init__(is_selected=is_selected, **kwargs)
+        self.name = name
+        self.fruits = fruits if fruits is not None else []
+        self.is_selected = is_selected
 
 
 class FruitItem(SelectableDataItem):
 
-    def __init__(self, **kwargs):
-        super(FruitItem, self).__init__(**kwargs)
-        self.name = kwargs.get('name', '')
-        self.serving_size = kwargs.get('Serving Size', '')
-        self.data = kwargs.get('data', [])
-        self.is_selected = kwargs.get('is_selected', False)
+    def __init__(self, is_selected=False, data=None, name='', **kwargs):
+        self.serving_size = kwargs.pop('Serving Size', '')
+        super(FruitItem, self).__init__(is_selected=is_selected, **kwargs)
+        self.name = name
+        self.data = data if data is not None else data
+        self.is_selected = is_selected
 
 
 def reset_to_defaults(data):
@@ -204,6 +204,7 @@ def reset_to_defaults(data):
         for obj in data:
             obj.is_selected = False
 
+
 category_data_items = \
     [CategoryItem(**fruit_categories[c]) for c in sorted(fruit_categories)]
 
@@ -407,18 +408,17 @@ class ListAdapterTestCase(unittest.TestCase):
 
         # Now unselect it with handle_selection().
         list_adapter.handle_selection(banana)
-        self.assertFalse(banana.is_selected)
 
-        # But, since we have allow_empty_selection=False, Apple will be
-        # reselected.
-        self.assertEqual(selection_observer.fruit_name, 'Apple')
+        # But, since we have allow_empty_selection=False, Banana will stay
+        # selected, behavior changed in #3672.
+        self.assertEqual(selection_observer.fruit_name, 'Banana')
 
         # Call count:
         #
         # Apple got selected initally (0), then unselected when Banana was
-        # selected (1). Then banana was unselected, causing reselection of
-        # Apple (3). len should be 1.
-        self.assertEqual(selection_observer.call_count, 3)
+        # selected (1). Then banana was unselected (2), and stayed selected.
+        # len should be 1.
+        self.assertEqual(selection_observer.call_count, 2)
         self.assertEqual(len(list_adapter.selection), 1)
 
 
