@@ -4,7 +4,7 @@ Settings
 
 .. versionadded:: 1.0.7
 
-This module is a complete and extensible framework for adding a
+This module provides a complete and extensible framework for adding a
 Settings interface to your application. By default, the interface uses
 a :class:`SettingsWithSpinner`, which consists of a
 :class:`~kivy.uix.spinner.Spinner` (top) to switch between individual
@@ -16,14 +16,14 @@ alternatives.
 
 A :class:`SettingsPanel` represents a group of configurable options. The
 :attr:`SettingsPanel.title` property is used by :class:`Settings` when a panel
-is added - it determines the name of the sidebar button. SettingsPanel controls
+is added: it determines the name of the sidebar button. SettingsPanel controls
 a :class:`~kivy.config.ConfigParser` instance.
 
 The panel can be automatically constructed from a JSON definition file: you
 describe the settings you want and corresponding sections/keys in the
 ConfigParser instance... and you're done!
 
-Settings are also integrated with the :class:`~kivy.app.App` class. Use
+Settings are also integrated into the :class:`~kivy.app.App` class. Use
 :meth:`Settings.add_kivy_panel` to configure the Kivy core settings in a panel.
 
 
@@ -44,7 +44,8 @@ To create a panel from a JSON-file, you need two things:
 
 You must create and handle the :class:`~kivy.config.ConfigParser`
 object. SettingsPanel will read the values from the associated
-ConfigParser instance. Make sure you have default values for all sections/keys
+ConfigParser instance. Make sure you have set default values (using
+:attr:`~kivy.config.ConfigParser.setdefaults`) for all the sections/keys
 in your JSON file!
 
 The JSON file contains structured information to describe the available
@@ -60,8 +61,7 @@ settings. Here is an example::
             "title": "Fullscreen",
             "desc": "Set the window in windowed or fullscreen",
             "section": "graphics",
-            "key": "fullscreen",
-            "true": "auto"
+            "key": "fullscreen"
         }
     ]
 
@@ -71,20 +71,23 @@ created and used for the setting - other keys are assigned to corresponding
 properties of that class.
 
     ============== =================================================
-     Type           Associated class
+    Type           Associated class
     -------------- -------------------------------------------------
     title          :class:`SettingTitle`
     bool           :class:`SettingBoolean`
     numeric        :class:`SettingNumeric`
     options        :class:`SettingOptions`
     string         :class:`SettingString`
-    path           :class:`SettingPath` (new from 1.1.0)
+    path           :class:`SettingPath`
     ============== =================================================
 
+    .. versionadded:: 1.1.0
+        Added :attr:`SettingPath` type
+
 In the JSON example above, the first element is of type "title". It will create
-a new instance of :class:`SettingTitle` and apply the rest of the key/value
+a new instance of :class:`SettingTitle` and apply the rest of the key-value
 pairs to the properties of that class, i.e. "title": "Windows" sets the
-:attr:`SettingTitle.title` property to "Windows".
+:attr:`~SettingsPanel.title` property of the panel to "Windows".
 
 To load the JSON example to a :class:`Settings` instance, use the
 :meth:`Settings.add_json_panel` method. It will automatically instantiate a
@@ -145,24 +148,26 @@ that displays a json settings panel with some way to switch between
 panels. An instance will be automatically created by :class:`Settings`.
 
 Interface widgets may be anything you like, but *must* have a method
-add_panel that recieves newly created json settings panels for the
+add_panel that receives newly created json settings panels for the
 interface to display. See the documentation for
 :class:`InterfaceWithSidebar` for more information. They may
 optionally dispatch an on_close event, for instance if a close button
 is clicked. This event is used by :class:`Settings` to trigger its own
 on_close event.
 
+For a complete, working example, please see
+:file:`kivy/examples/settings/main.py`.
+
 '''
 
 __all__ = ('Settings', 'SettingsPanel', 'SettingItem', 'SettingString',
            'SettingPath', 'SettingBoolean', 'SettingNumeric', 'SettingOptions',
            'SettingTitle', 'SettingsWithSidebar', 'SettingsWithSpinner',
            'SettingsWithTabbedPanel', 'SettingsWithNoMenu',
-           'InterfaceWithSidebar', 'ContentPanel')
+           'InterfaceWithSidebar', 'ContentPanel', 'MenuSidebar')
 
 import json
 import os
-from kivy.compat import string_types
 from kivy.factory import Factory
 from kivy.metrics import dp
 from kivy.config import ConfigParser
@@ -366,7 +371,7 @@ class SettingString(SettingItem):
     def on_panel(self, instance, value):
         if value is None:
             return
-        self.bind(on_release=self._create_popup)
+        self.fbind('on_release', self._create_popup)
 
     def _dismiss(self, *largs):
         if self.textinput:
@@ -440,10 +445,29 @@ class SettingPath(SettingItem):
     defaults to None.
     '''
 
+    show_hidden = BooleanProperty(False)
+    '''Whether to show 'hidden' filenames. What that means is
+    operating-system-dependent.
+
+    :attr:`show_hidden` is an :class:`~kivy.properties.BooleanProperty` and
+    defaults to False.
+
+    .. versionadded:: 1.10.0
+    '''
+
+    dirselect = BooleanProperty(True)
+    '''Whether to allow selection of directories.
+
+    :attr:`dirselect` is a :class:`~kivy.properties.BooleanProperty` and
+    defaults to True.
+
+    .. versionadded:: 1.10.0
+    '''
+
     def on_panel(self, instance, value):
         if value is None:
             return
-        self.bind(on_release=self._create_popup)
+        self.fbind('on_release', self._create_popup)
 
     def _dismiss(self, *largs):
         if self.textinput:
@@ -470,10 +494,11 @@ class SettingPath(SettingItem):
             width=popup_width)
 
         # create the filechooser
+        initial_path = self.value or os.getcwd()
         self.textinput = textinput = FileChooserListView(
-            path=self.value, size_hint=(1, 1), dirselect=True)
+            path=initial_path, size_hint=(1, 1),
+            dirselect=self.dirselect, show_hidden=self.show_hidden)
         textinput.bind(on_path=self._validate)
-        self.textinput = textinput
 
         # construct the content
         content.add_widget(textinput)
@@ -540,7 +565,7 @@ class SettingOptions(SettingItem):
     def on_panel(self, instance, value):
         if value is None:
             return
-        self.bind(on_release=self._create_popup)
+        self.fbind('on_release', self._create_popup)
 
     def _set_option(self, instance):
         self.value = instance.text
@@ -580,6 +605,8 @@ class SettingTitle(Label):
 
     title = Label.text
 
+    panel = ObjectProperty(None)
+
 
 class SettingsPanel(GridLayout):
     '''This class is used to contruct panel settings, for use with a
@@ -602,7 +629,8 @@ class SettingsPanel(GridLayout):
     '''
 
     def __init__(self, **kwargs):
-        kwargs.setdefault('cols', 1)
+        if 'cols' not in kwargs:
+            self.cols = 1
         super(SettingsPanel, self).__init__(**kwargs)
 
     def on_config(self, instance, value):
@@ -683,16 +711,17 @@ class InterfaceWithSidebar(BoxLayout):
         display. Any replacement for ContentPanel *must* implement
         this method.
 
-        :param panel: A :class:`SettingsPanel`. It should be stored
-                      and the interface should provide a way to switch
-                      between panels.
+        :Parameters:
+            `panel`: :class:`SettingsPanel`
+                It should be stored and the interface should provide a way to
+                switch between panels.
+            `name`:
+                The name of the panel as a string. It may be used to represent
+                the panel but isn't necessarily unique.
+            `uid`:
+                A unique int identifying the panel. It should be used to
+                identify and switch between panels.
 
-        :param name: The name of the panel as a string. It
-                     may be used to represent the panel but isn't necessarily
-                     unique.
-
-        :param uid: A unique int identifying the panel. It should be
-                    used to identify and switch between panels.
         '''
         self.menu.add_item(name, uid)
         self.content.add_panel(panel, name, uid)
@@ -717,7 +746,7 @@ class InterfaceWithSpinner(BoxLayout):
     '''(internal) A reference to the sidebar menu widget.
 
     :attr:`menu` is an :class:`~kivy.properties.ObjectProperty` and
-    defauls to None.
+    defaults to None.
     '''
 
     content = ObjectProperty()
@@ -739,16 +768,16 @@ class InterfaceWithSpinner(BoxLayout):
         display. Any replacement for ContentPanel *must* implement
         this method.
 
-        :param panel: A :class:`SettingsPanel`. It should be stored
-                      and the interface should provide a way to switch
-                      between panels.
-
-        :param name: The name of the panel as a string. It
-                     may be used to represent the panel but may not
-                     be unique.
-
-        :param uid: A unique int identifying the panel. It should be
-                    used to identify and switch between panels.
+        :Parameters:
+            `panel`: :class:`SettingsPanel`
+                It should be stored and the interface should provide a way to
+                switch between panels.
+            `name`:
+                The name of the panel as a string. It may be used to represent
+                the panel but may not be unique.
+            `uid`:
+                A unique int identifying the panel. It should be used to
+                identify and switch between panels.
 
         '''
         self.content.add_panel(panel, name, uid)
@@ -804,14 +833,15 @@ class ContentPanel(ScrollView):
         display. Any replacement for ContentPanel *must* implement
         this method.
 
-        :param panel: A :class:`SettingsPanel`. It should be stored
-                      and displayed when requested.
-
-        :param name: The name of the panel as a string. It
-                     may be used to represent the panel.
-
-        :param uid: A unique int identifying the panel. It should be
-                    stored and used to identify panels when switching.
+        :Parameters:
+            `panel`: :class:`SettingsPanel`
+                It should be stored and displayed when requested.
+            `name`:
+                The name of the panel as a string. It may be used to represent
+                the panel.
+            `uid`:
+                A unique int identifying the panel. It should be stored and
+                used to identify panels when switching.
 
         '''
         self.panels[uid] = panel
@@ -822,9 +852,12 @@ class ContentPanel(ScrollView):
         '''The uid of the currently displayed panel. Changing this will
         automatically change the displayed panel.
 
-        :param uid: A panel uid. It should be used to retrieve and
-                    display a settings panel that has previously been
-                    added with :meth:`add_panel`.
+        :Parameters:
+            `uid`:
+                A panel uid. It should be used to retrieve and display
+                a settings panel that has previously been added with
+                :meth:`add_panel`.
+
         '''
         uid = self.current_uid
         if uid in self.panels:
@@ -853,7 +886,7 @@ class Settings(BoxLayout):
 
     :Events:
         `on_config_change`: ConfigParser instance, section, key, value
-            Fired when section/key/value of a ConfigParser changes.
+            Fired when the section's key-value pair of a ConfigParser changes.
 
             .. warning:
 
@@ -880,7 +913,7 @@ class Settings(BoxLayout):
 
     :attr:`interface_cls` is an
     :class:`~kivy.properties.ObjectProperty` and defaults to
-    :class`InterfaceWithSidebar`.
+    :class:`InterfaceWithSidebar`.
 
     .. versionchanged:: 1.8.0
         If you set a string, the :class:`~kivy.factory.Factory` will be used to
@@ -963,7 +996,7 @@ class Settings(BoxLayout):
 
         for setting in data:
             # determine the type and the class to use
-            if not 'type' in setting:
+            if 'type' not in setting:
                 raise ValueError('One setting are missing the "type" element')
             ttype = setting['type']
             cls = self._types.get(ttype)
@@ -1148,7 +1181,7 @@ class MenuSidebar(FloatLayout):
     :attr:`~ContentPanel.current_uid` of a :class:`ContentPanel`.
 
     :attr:`selected_uid` is a
-    :class`~kivy.properties.NumericProperty` and defaults to 0.
+    :class:`~kivy.properties.NumericProperty` and defaults to 0.
 
     '''
 
@@ -1172,12 +1205,14 @@ class MenuSidebar(FloatLayout):
     def add_item(self, name, uid):
         '''This method is used to add new panels to the menu.
 
-        :param name: The name (a string) of the panel. It should be
-                     used to represent the panel in the menu.
-
-        :param uid: The name (an int) of the panel. It should be used
-                    internally to represent the panel and used to set
-                    self.selected_uid when the panel is changed.
+        :Parameters:
+            `name`:
+                The name (a string) of the panel. It should be used
+                to represent the panel in the menu.
+            `uid`:
+                The name (an int) of the panel. It should be used internally
+                to represent the panel and used to set self.selected_uid when
+                the panel is changed.
 
         '''
 
